#include "RooGlobalFunc.h"
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooHistPdf.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "RooPolyVar.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "TChain.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TLegend.h"
#include "TFile.h"
#include "RooDstD0BG.h"
#include "TLatex.h"
#include "myRooPolBG.cpp"
#include "myRooJohnsonSU.cpp"

using namespace RooFit ;
using namespace std;

void simfit_belle_mc_wt_hist_pdf()
{
  TFile *f = new TFile("component_wise_trees_for_fit_kk_belle_mc_4Sonly.root");

  TTree *tree = (TTree*)f->Get("complete");
  //Variables for 2Dfit
  RooRealVar mD0pi("mD0pi","", 2.005, 2.025, "");
  RooRealVar mKK("mKK","", 1.75, 2.05, "");

  //Variable used to separate the total sample into two (should be there in the tree)
  RooRealVar pisoft("pisoft","",-2,2,"");
  RooRealVar weights("weights","", 0, 1.5, "");

  //Separated datasets
  RooDataSet *dataxy_d0 = new RooDataSet("data_d0","data_d0",tree,RooArgSet(mKK,mD0pi,pisoft,weights),"pisoft==1");
  RooDataSet *dataxy_d0bar = new RooDataSet("data_d0bar","data_d0bar",tree,RooArgSet(mKK,mD0pi,pisoft,weights),"pisoft==-1"); 

  /////////////////////////////////////////////////////

  //For RooHistPDF for signal (gamma)
  TTree *tree_mult = (TTree*)f->Get("multibody");
  // RooRealVar log_min_significance_sig("log_min_significance_sig","", -6, 6.7, "");
  RooDataSet data("data","data", tree_mult, RooArgSet(mKK));
  mKK.setBins(100);
  // Create a binned dataset with 20 bins and 500 events
  RooDataHist* hist_mult_KK = data.binnedClone() ;
  RooHistPdf mult_mKK("mult_mKK","mult_mKK",mKK, *hist_mult_KK,0);  

 /*-----------------------S I G N A L-------------------------*/
   //d0pi
  RooRealVar f_J_sig_d0pi("f_{Jsig_d0pi}", "rel_frac_j",4.42029e-01);
  RooRealVar A_fJ_sig_d0pi("A_{fJ_sig_d0pi}","",0);
  RooFormulaVar f_J_sig_d0pi_d0("f_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(f_J_sig_d0pi,A_fJ_sig_d0pi));
  RooFormulaVar f_J_sig_d0pi_d0bar("f_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(f_J_sig_d0pi,A_fJ_sig_d0pi));

  RooRealVar mu_J_sig_d0pi("#mu_{Jsig_d0pi}","mean_johnson",2.01022e+00);
  RooRealVar A_mu_J_sig_d0pi("A_{mu_J_sig_d0pi}","",0);
  RooFormulaVar mu_J_sig_d0pi_d0("mu_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(mu_J_sig_d0pi, A_mu_J_sig_d0pi));
  RooFormulaVar mu_J_sig_d0pi_d0bar("mu_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(mu_J_sig_d0pi, A_mu_J_sig_d0pi));

  RooRealVar sigma_J_sig_d0pi("#sigma_{Jsig_d0pi}", "sigma_johnson",2.17443e-04);
  RooRealVar A_sigma_J_sig_d0pi("A_{sigma_J_sig_d0pi}","",0);
  RooFormulaVar sigma_J_sig_d0pi_d0("sigma_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_J_sig_d0pi, A_sigma_J_sig_d0pi));
  RooFormulaVar sigma_J_sig_d0pi_d0bar("sigma_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_sig_d0pi, A_sigma_J_sig_d0pi));

  RooRealVar gamma_J_sig_d0pi("#gamma_{Jsig_d0pi}", "gamma", -2.12686e-01);
  RooRealVar A_gamma_J_sig_d0pi("A_{gamma_J_sig_d0pi}","",0);
  RooFormulaVar gamma_J_sig_d0pi_d0("gamma_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(gamma_J_sig_d0pi, A_gamma_J_sig_d0pi));
  RooFormulaVar gamma_J_sig_d0pi_d0bar("gamma_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_sig_d0pi, A_gamma_J_sig_d0pi));

  RooRealVar delta_J_sig_d0pi("#delta_{Jsig_d0pi}", "delta", 9.20435e-01);
  RooRealVar A_delta_J_sig_d0pi("A_{delta_J_sig_d0pi}","",0);
  RooFormulaVar delta_J_sig_d0pi_d0("delta_J_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(delta_J_sig_d0pi, A_delta_J_sig_d0pi));
  RooFormulaVar delta_J_sig_d0pi_d0bar("delta_J_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(delta_J_sig_d0pi, A_delta_J_sig_d0pi));

  myRooJohnsonSU johnson_sig_d0pi_d0("johnson_sig_d0pi_d0","johnson_sig_d0", mD0pi, mu_J_sig_d0pi_d0, sigma_J_sig_d0pi_d0, delta_J_sig_d0pi_d0, gamma_J_sig_d0pi_d0);
  myRooJohnsonSU johnson_sig_d0pi_d0bar("johnson_sig_d0pi_d0bar","johnson_sig_d0bar", mD0pi, mu_J_sig_d0pi_d0bar, sigma_J_sig_d0pi_d0bar, delta_J_sig_d0pi_d0bar, gamma_J_sig_d0pi_d0bar);
  
  RooRealVar f_G1_sig_d0pi("f_{G1sig_d0pi}", "frac_gauss1",2.93118e-01);
  RooRealVar A_f_G1_sig_d0pi("A_{f_G1_sig_d0pi}","",0);
  RooFormulaVar f_G1_sig_d0pi_d0("f_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(f_G1_sig_d0pi, A_f_G1_sig_d0pi));
  RooFormulaVar f_G1_sig_d0pi_d0bar("f_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(f_G1_sig_d0pi, A_f_G1_sig_d0pi));

  RooRealVar mu_G1_sig_d0pi("#mu_{G1sig_d0pi}", "mean_gauss1",2.01025e+00);
  RooRealVar A_mu_G1_sig_d0pi("A_{mu_G1_sig_d0pi}","",0);
  RooFormulaVar mu_G1_sig_d0pi_d0("mu_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(mu_G1_sig_d0pi, A_mu_G1_sig_d0pi));
  RooFormulaVar mu_G1_sig_d0pi_d0bar("mu_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(mu_G1_sig_d0pi, A_mu_G1_sig_d0pi));

  RooRealVar sigma_G1_sig_d0pi("#sigma_{G1sig_d0pi}", "sigma_gauss1",3.93745e-04);
  RooRealVar A_sigma_G1_sig_d0pi("A_{sigma_G1_sig_d0pi}","",0);
  RooFormulaVar sigma_G1_sig_d0pi_d0("sigma_G1_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_G1_sig_d0pi, A_sigma_G1_sig_d0pi));
  RooFormulaVar sigma_G1_sig_d0pi_d0bar("sigma_G1_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G1_sig_d0pi, A_sigma_G1_sig_d0pi));

  RooRealVar sigma_G2_sig_d0pi("#sigma_{G2sig_d0pi}", "sigma_gauss2",2.26930e-04);
  RooRealVar A_sigma_G2_sig_d0pi("A_{sigma_G2_sig_d0pi}","",0);
  RooFormulaVar sigma_G2_sig_d0pi_d0("sigma_G2_sig_d0pi_d0","","(@0*(1 + @1))",RooArgList(sigma_G2_sig_d0pi, A_sigma_G2_sig_d0pi));
  RooFormulaVar sigma_G2_sig_d0pi_d0bar("sigma_G2_sig_d0pi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G2_sig_d0pi, A_sigma_G2_sig_d0pi));

  RooGaussian gauss1_sig_d0pi_d0("gauss1_sig_d0pi_d0", "1st gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0, sigma_G1_sig_d0pi_d0);
  RooGaussian gauss1_sig_d0pi_d0bar("gauss1_sig_d0pi_d0bar", "1st gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0bar, sigma_G1_sig_d0pi_d0bar);
  RooGaussian gauss2_sig_d0pi_d0("gauss2_sig_d0pi_d0", "2nd gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0, sigma_G2_sig_d0pi_d0);
  RooGaussian gauss2_sig_d0pi_d0bar("gauss2_sig_d0pi_d0bar", "2nd gaussian PDF", mD0pi, mu_G1_sig_d0pi_d0bar, sigma_G2_sig_d0pi_d0bar);
  
  RooAddPdf doublegaussian_sig_d0pi_d0("doublegaussian_sig_d0", "doublegaussian_sig", gauss1_sig_d0pi_d0, gauss2_sig_d0pi_d0, f_G1_sig_d0pi_d0);
  RooAddPdf doublegaussian_sig_d0pi_d0bar("doublegaussian_sig_d0bar", "doublegaussian_sig", gauss1_sig_d0pi_d0bar, gauss2_sig_d0pi_d0bar, f_G1_sig_d0pi_d0bar);
  
  RooAddPdf sig_mD0pi_d0("sig_mD0pi_d0","sig_mD0pi_d0", johnson_sig_d0pi_d0, doublegaussian_sig_d0pi_d0, f_J_sig_d0pi_d0);
  RooAddPdf sig_mD0pi_d0bar("sig_mD0pi_d0bar","sig_mD0pi_d0bar", johnson_sig_d0pi_d0bar, doublegaussian_sig_d0pi_d0bar, f_J_sig_d0pi_d0bar);
  
  //kk
  RooRealVar f_J_sig_dz("f_{J_sig_dz}", "relative_frac_j",9.80896e-01);
  RooRealVar A_f_J_sig_dz("A_{f_J_sig_dz}","",0);
  RooFormulaVar f_J_sig_dz_d0("f_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(f_J_sig_dz, A_f_J_sig_dz));
  RooFormulaVar f_J_sig_dz_d0bar("f_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(f_J_sig_dz, A_f_J_sig_dz));

  RooRealVar mu_J_sig_dz("#mu_{J_sig_dz}","mean_johnson",1.86437e+00);
  RooRealVar A_mu_J_sig_dz("A_{mu_J_sig_dz}","",0);
  RooFormulaVar mu_J_sig_dz_d0("mu_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(mu_J_sig_dz, A_mu_J_sig_dz));
  RooFormulaVar mu_J_sig_dz_d0bar("mu_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(mu_J_sig_dz, A_mu_J_sig_dz));

  RooRealVar sigma_J_sig_dz("#sigma_{J_sig_dz}", "sigma_johnson", 9.55229e-03);
  RooRealVar A_sigma_J_sig_dz("A_{sigma_J_sig_dz}","",0);
  RooFormulaVar sigma_J_sig_dz_d0("sigma_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_J_sig_dz, A_sigma_J_sig_dz));
  RooFormulaVar sigma_J_sig_dz_d0bar("sigma_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_sig_dz, A_sigma_J_sig_dz));

  RooRealVar gamma_J_sig_dz("#gamma_{J_sig_dz}", "gamma",-1.26485e-01);
  RooRealVar A_gamma_J_sig_dz("A_{gamma_J_sig_dz}","",0);
  RooFormulaVar gamma_J_sig_dz_d0("gamma_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(gamma_J_sig_dz, A_gamma_J_sig_dz));
  RooFormulaVar gamma_J_sig_dz_d0bar("gamma_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_sig_dz, A_gamma_J_sig_dz));

  RooRealVar delta_J_sig_dz("#delta_{J_sig_dz}", "delta",1.97179e+00);
  RooRealVar A_delta_J_sig_dz("A_{delta_J_sig_dz}","",0);
  RooFormulaVar delta_J_sig_dz_d0("delta_J_sig_dz_d0","","(@0*(1 + @1))",RooArgList(delta_J_sig_dz, A_delta_J_sig_dz));
  RooFormulaVar delta_J_sig_dz_d0bar("delta_J_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(delta_J_sig_dz, A_delta_J_sig_dz));

  myRooJohnsonSU johnson_sig_dz_d0("johnson_sig_dz_d0", "johnson_sig_dz_d0", mKK, mu_J_sig_dz_d0, sigma_J_sig_dz_d0, delta_J_sig_dz_d0, gamma_J_sig_dz_d0);
  myRooJohnsonSU johnson_sig_dz_d0bar("johnson_sig_dz_d0bar", "johnson_sig_dz_d0bar", mKK, mu_J_sig_dz_d0bar, sigma_J_sig_dz_d0bar, delta_J_sig_dz_d0bar, gamma_J_sig_dz_d0bar);
  
  RooRealVar f_G1_sig_dz("f_{G1}_sig_dz", "relative_frac_g",1.75958e-01);
  RooRealVar A_f_G1_sig_dz("A_{f_G1_sig_dz}","",0);
  RooFormulaVar f_G1_sig_dz_d0("f_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(f_G1_sig_dz, A_f_G1_sig_dz));
  RooFormulaVar f_G1_sig_dz_d0bar("f_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(f_G1_sig_dz, A_f_G1_sig_dz));

  RooRealVar mu_G1_sig_dz("#mu_{G1}_sig_dz", "mean_gauss1",1.83949e+00);
  RooRealVar A_mu_G1_sig_dz("A_{mu_G1_sig_dz}","",0);
  RooFormulaVar mu_G1_sig_dz_d0("mu_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(mu_G1_sig_dz, A_mu_G1_sig_dz));
  RooFormulaVar mu_G1_sig_dz_d0bar("mu_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(mu_G1_sig_dz, A_mu_G1_sig_dz));

  RooRealVar sigma_G1_sig_dz("#sigma_{G1}_sig_dz", "sigma_gauss1",7.21444e-03);
  RooRealVar A_sigma_G1_sig_dz("A_{sigma_G1_sig_dz}","",0);
  RooFormulaVar sigma_G1_sig_dz_d0("sigma_G1_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_G1_sig_dz, A_sigma_G1_sig_dz));
  RooFormulaVar sigma_G1_sig_dz_d0bar("sigma_G1_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_G1_sig_dz, A_sigma_G1_sig_dz));
  
  RooRealVar sigma_G2_sig_dz("#sigma_{G2}_sig_dz", "sigma_gauss2",4.42093e-02);
  RooRealVar A_sigma_G2_sig_dz("A_{sigma_G2_sig_dz}","",0);
  RooFormulaVar sigma_G2_sig_dz_d0("sigma_G2_sig_dz_d0","","(@0*(1 + @1))",RooArgList(sigma_G2_sig_dz, A_sigma_G2_sig_dz));
  RooFormulaVar sigma_G2_sig_dz_d0bar("sigma_G2_sig_dz_d0bar","","(@0*(1- @1))",RooArgList(sigma_G2_sig_dz, A_sigma_G2_sig_dz));

  RooGaussian gauss1_sig_dz_d0("gauss1_sig_dz_d0", "1st gaussian PDF", mKK, mu_G1_sig_dz_d0, sigma_G1_sig_dz_d0);
  RooGaussian gauss1_sig_dz_d0bar("gauss1_sig_dz_d0bar", "1st gaussian PDF", mKK, mu_G1_sig_dz_d0bar, sigma_G1_sig_dz_d0bar);
  
  RooGaussian gauss2_sig_dz_d0("gauss2_sig_dz_d0", "2nd gaussian PDF", mKK, mu_G1_sig_dz_d0, sigma_G2_sig_dz_d0);
  RooGaussian gauss2_sig_dz_d0bar("gauss2_sig_dz_d0bar", "2nd gaussian PDF", mKK, mu_G1_sig_dz_d0bar, sigma_G2_sig_dz_d0bar);

  RooAddPdf doublegaussian_sig_dz_d0("doublegaussian_sig_dz_d0", "doublegaussian_sig_dz_d0", gauss1_sig_dz_d0, gauss2_sig_dz_d0, f_G1_sig_dz_d0);
  RooAddPdf doublegaussian_sig_dz_d0bar("doublegaussian_sig_dz_d0bar", "doublegaussian_sig_dz_d0bar", gauss1_sig_dz_d0bar, gauss2_sig_dz_d0bar, f_G1_sig_dz_d0bar);
  
  RooAddPdf sig_mKK_d0("sig_mKK_d0","sig_mKK_d0", johnson_sig_dz_d0, doublegaussian_sig_dz_d0, f_J_sig_dz_d0);
  RooAddPdf sig_mKK_d0bar("sig_mKK_d0bar","sig_mKK_d0bar", johnson_sig_dz_d0bar, doublegaussian_sig_dz_d0bar, f_J_sig_dz_d0bar);
  
  //signal peak
  RooProdPdf signal_peak_d0("signal_peak_d0", "signal_peak_d0", sig_mKK_d0, sig_mD0pi_d0);
  RooProdPdf signal_peak_d0bar("signal_peak_d0bar", "signal_peak_d0bar", sig_mKK_d0bar, sig_mD0pi_d0bar);
  
  //random pion (common for all)
  RooRealVar m0("m0", "no data beyond this", 2.00441);
  RooRealVar kappa("C", "C", 1.59953e-03,-1e6,1e6);
  RooRealVar alpha("A", "A", 3.75892e+01,0,1e6);
  RooRealVar beta("B",  "B" ,0);
  RooDstD0BG bkg_mD0pi("bkg_mD0pi","bkg_mD0pi", mD0pi, m0, kappa, alpha, beta);
  
  //signal & random pion
  RooProdPdf signal_rnd_d0("signal_rnd_d0", "signal_rnd_d0", sig_mKK_d0, bkg_mD0pi);
  RooProdPdf signal_rnd_d0bar("signal_rnd_d0bar", "signal_rnd_d0bar", sig_mKK_d0bar, bkg_mD0pi);

  /*-----------------------K Pi-------------------------*/

  //kk
  RooRealVar f1_d0kpi("f_{G}d0kpi", "relative fraction gaussian",3.14595e-01);
  RooRealVar A_f1_d0kpi("A_{f1_d0kpi}","",0);
  RooFormulaVar f1_d0kpi_d0("f1_d0kpi_d0","","(@0*(1 + @1))",RooArgList(f1_d0kpi, A_f1_d0kpi));
  RooFormulaVar f1_d0kpi_d0bar("f1_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(f1_d0kpi, A_f1_d0kpi));

  RooRealVar mu_G_d0kpi("#mu_{G}d0kpi","mean_gaus",1.94487e+00);
  RooRealVar A_mu_G_d0kpi("A_{mu_G_d0kpi}","",0);
  RooFormulaVar mu_G_d0kpi_d0("mu_G_d0kpi_d0","","(@0*(1 + @1))",RooArgList(mu_G_d0kpi, A_mu_G_d0kpi));
  RooFormulaVar mu_G_d0kpi_d0bar("mu_G_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(mu_G_d0kpi, A_mu_G_d0kpi));

  RooRealVar sigma_G_d0kpi("#sigma_{G}d0kpi", "sigma_gaus",9.49976e-03);
  RooRealVar A_sigma_G_d0kpi("A_{sigma_G_d0kpi}","",0);
  RooFormulaVar sigma_G_d0kpi_d0("sigma_G_d0kpi_d0","","(@0*(1 + @1))",RooArgList(sigma_G_d0kpi, A_sigma_G_d0kpi));
  RooFormulaVar sigma_G_d0kpi_d0bar("sigma_G_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(sigma_G_d0kpi, A_sigma_G_d0kpi));

  RooGaussian gauss_d0kpi_d0("gauss_d0kpi_d0", "gauss component", mKK, mu_G_d0kpi_d0, sigma_G_d0kpi_d0);
  RooGaussian gauss_d0kpi_d0bar("gauss_d0kpi_d0bar", "gauss component", mKK, mu_G_d0kpi_d0bar, sigma_G_d0kpi_d0bar);
  
  RooRealVar gamma_J_d0kpi("#gamma_{J}d0kpi", "gamma",-1.55916e+00);
  RooRealVar A_gamma_J_d0kpi("A_{gamma_J_d0kpi}","",0);
  RooFormulaVar gamma_J_d0kpi_d0("gamma_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(gamma_J_d0kpi, A_gamma_J_d0kpi));
  RooFormulaVar gamma_J_d0kpi_d0bar("gamma_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(gamma_J_d0kpi, A_gamma_J_d0kpi));

  RooRealVar delta_J_d0kpi("#delta_{J}d0kpi", "delta",4.05187e-01);
  RooRealVar A_delta_J_d0kpi("A_{delta_J_d0kpi}","",0);
  RooFormulaVar delta_J_d0kpi_d0("delta_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(delta_J_d0kpi, A_delta_J_d0kpi));
  RooFormulaVar delta_J_d0kpi_d0bar("delta_K_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(delta_J_d0kpi, A_delta_J_d0kpi));

  RooRealVar mu_J_d0kpi("#mu_{J}_d0kpi", "mean",1.96219e+00);
  RooRealVar A_mu_J_d0kpi("A_{mu_J_d0kpi}","",0);
  RooFormulaVar mu_J_d0kpi_d0("mu_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(mu_J_d0kpi, A_mu_J_d0kpi));
  RooFormulaVar mu_J_d0kpi_d0bar("mu_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(mu_J_d0kpi, A_mu_J_d0kpi));

  RooRealVar sigma_J_d0kpi("#sigma_{J}_d0kpi", "sigma",1.20908e-02);
  RooRealVar A_sigma_J_d0kpi("A_{sigma_J_d0kpi}","",0);
  RooFormulaVar sigma_J_d0kpi_d0("sigma_J_d0kpi_d0","","(@0*(1 + @1))",RooArgList(sigma_J_d0kpi, A_sigma_J_d0kpi));
  RooFormulaVar sigma_J_d0kpi_d0bar("sigma_J_d0kpi_d0bar","","(@0*(1- @1))",RooArgList(sigma_J_d0kpi, A_sigma_J_d0kpi));

  myRooJohnsonSU johnson_d0kpi_d0("johnson_d0kpi_d0", "Johnson PDF", mKK, mu_J_d0kpi_d0, sigma_J_d0kpi_d0, delta_J_d0kpi_d0, gamma_J_d0kpi_d0);
  myRooJohnsonSU johnson_d0kpi_d0bar("johnson_d0kpi_d0bar", "Johnson PDF", mKK, mu_J_d0kpi_d0bar, sigma_J_d0kpi_d0bar, delta_J_d0kpi_d0bar, gamma_J_d0kpi_d0bar);
  
  RooAddPdf kpi_mKK_d0("kpi_mKK_d0","kpi_mKK_d0",gauss_d0kpi_d0,johnson_d0kpi_d0,f1_d0kpi_d0);
  RooAddPdf kpi_mKK_d0bar("kpi_mKK_d0bar","kpi_mKK_d0bar",gauss_d0kpi_d0bar,johnson_d0kpi_d0bar,f1_d0kpi_d0bar);
  
  //kpi peaking background
  RooProdPdf kpi_peak_d0("kpi_peak_d0", "kpi_peak_d0", kpi_mKK_d0, sig_mD0pi_d0);//as the shape of m(d0pi) is same as signal
  RooProdPdf kpi_peak_d0bar("kpi_peak_d0bar", "kpi_peak_d0bar", kpi_mKK_d0bar, sig_mD0pi_d0bar);//as the shape of m(d0pi) is same as signal
  
  //kpi & random pion
  RooProdPdf kpi_rnd_d0("kpi_rnd_d0", "kpi_rnd_d0", kpi_mKK_d0, bkg_mD0pi);
  RooProdPdf kpi_rnd_d0bar("kpi_rnd_d0bar", "kpi_rnd_d0bar", kpi_mKK_d0bar, bkg_mD0pi);

  /*-----------------------M U L T I B O D Y-------------------------*/
  
  RooRealVar f_J1_mult("f_{J1_mult}", "",2.23696e-01);
  RooRealVar A_f_J1_mult("A_{f_J1_mult}","",0);
  RooFormulaVar f_J1_mult_d0("f_J1_mult_d0","","(@0*(1 + @1))",RooArgList(f_J1_mult,  A_f_J1_mult));
  RooFormulaVar f_J1_mult_d0bar("f_J1_mult_d0","","(@0*(1- @1))",RooArgList(f_J1_mult, A_f_J1_mult));
 
  RooRealVar mu_J1_mult("#mu_{J1_mult}","mean_johnson",2.00642e+00);
  RooRealVar A_mu_J1_mult("A_{mu_J1_mult}","",0);
  RooFormulaVar mu_J1_mult_d0("mu_J1_mult_d0","","(@0*(1 + @1))",RooArgList(mu_J1_mult, A_mu_J1_mult));
  RooFormulaVar mu_J1_mult_d0bar("mu_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(mu_J1_mult, A_mu_J1_mult));

  RooRealVar lambda_J1_mult("#lambda_{J1_mult}", "sigma_johnson",9.58128e-04);
  RooRealVar A_lambda_J1_mult("A_{lambda_J1_mult}","",0);
  RooFormulaVar lambda_J1_mult_d0("lambda_J1_mult_d0","","(@0*(1 + @1))",RooArgList(lambda_J1_mult, A_lambda_J1_mult));
  RooFormulaVar lambda_J1_mult_d0bar("lambda_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(lambda_J1_mult, A_lambda_J1_mult));

  RooRealVar gamma_J1_mult("#gamma_{J1_mult}", "gamma",-2.39994e+00);
  RooRealVar A_gamma_J1_mult("A_{gamma_J1_mult}","",0);
  RooFormulaVar gamma_J1_mult_d0("gamma_J1_mult_d0","","(@0*(1 + @1))",RooArgList(gamma_J1_mult, A_gamma_J1_mult));
  RooFormulaVar gamma_J1_mult_d0bar("gamma_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(gamma_J1_mult, A_gamma_J1_mult));

  RooRealVar delta_J1_mult("#delta_{J1_mult}", "delta",2.11240e-01);
  RooRealVar A_delta_J1_mult("A_{delta_J1_mult}","",0);
  RooFormulaVar delta_J1_mult_d0("delta_J1_mult_d0","","(@0*(1 + @1))",RooArgList(delta_J1_mult, A_delta_J1_mult));
  RooFormulaVar delta_J1_mult_d0bar("delta_J1_mult_d0bar","","(@0*(1- @1))",RooArgList(delta_J1_mult, A_delta_J1_mult));
 
  myRooJohnsonSU johnson1_mult_d0("johnson1_d0", "Johnson PDF", mD0pi, mu_J1_mult_d0, lambda_J1_mult_d0, delta_J1_mult_d0, gamma_J1_mult_d0);
  myRooJohnsonSU johnson1_mult_d0bar("johnson1_d0bar", "Johnson PDF", mD0pi, mu_J1_mult_d0bar, lambda_J1_mult_d0bar, delta_J1_mult_d0bar, gamma_J1_mult_d0bar);

  RooRealVar mu_J2_mult("#mu_{J2_mult}","mean_johnson",2.00619e+00);
  RooRealVar A_mu_J2_mult("A_{mu_J2_mult}","",0);
  RooFormulaVar mu_J2_mult_d0("mu_J2_mult_d0","","(@0*(1 + @1))",RooArgList(mu_J2_mult, A_mu_J2_mult));
  RooFormulaVar mu_J2_mult_d0bar("mu_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(mu_J2_mult, A_mu_J2_mult));

  RooRealVar lambda_J2_mult("#lambda_{J2_mult}", "sigma_johnson",3.71002e-03);
  RooRealVar A_lambda_J2_mult("A_{lambda_J2_mult}","",0);
  RooFormulaVar lambda_J2_mult_d0("lambda_J2_mult_d0","","(@0*(1 + @1))",RooArgList(lambda_J2_mult, A_lambda_J2_mult));
  RooFormulaVar lambda_J2_mult_d0bar("lambda_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(lambda_J2_mult, A_lambda_J2_mult));

  RooRealVar delta_J2_mult("#delta_{J2_mult}", "delta",2.09630e+00);
  RooRealVar A_delta_J2_mult("A_{delta_J2_mult}","",0);
  RooFormulaVar delta_J2_mult_d0("delta_J2_mult_d0","","(@0*(1 + @1))",RooArgList(delta_J2_mult, A_delta_J2_mult));
  RooFormulaVar delta_J2_mult_d0bar("delta_J2_mult_d0bar","","(@0*(1- @1))",RooArgList(delta_J2_mult, A_delta_J2_mult));

  myRooJohnsonSU johnson2_mult_d0("johnson2_d0", "Johnson PDF", mD0pi, mu_J2_mult_d0, lambda_J2_mult_d0, delta_J2_mult_d0, gamma_J1_mult_d0);
  myRooJohnsonSU johnson2_mult_d0bar("johnson2_d0bar", "Johnson PDF", mD0pi, mu_J2_mult_d0bar, lambda_J2_mult_d0bar, delta_J2_mult_d0bar, gamma_J1_mult_d0bar);
   
  RooAddPdf mult_mD0pi_d0("mult_mD0pi_d0", "mult_mD0pi_d0", johnson1_mult_d0, johnson2_mult_d0, f_J1_mult_d0);
  RooAddPdf mult_mD0pi_d0bar("mult_mD0pi_d0bar", "mult_mD0pi_d0bar", johnson1_mult_d0bar, johnson2_mult_d0bar, f_J1_mult_d0bar);
  //kk
  // RooRealVar a1_mult("a_{1_mult}", "A1", -13.3508);
  // RooRealVar A_a1_mult("A_{a1_mult}","",0);
  // RooFormulaVar a1_mult_d0("a1_mult_d0","","(@0*(1 + @1))",RooArgList(a1_mult, A_a1_mult));
  // RooFormulaVar a1_mult_d0bar("a1_mult_d0bar","","(@0*(1- @1))",RooArgList(a1_mult, A_a1_mult));
  
  // RooExponential mult_mKK_d0("mult_mKK_d0", "mult_mKK_d0", mKK, a1_mult_d0);  
  // RooExponential mult_mKK_d0bar("mult_mKK_d0bar", "mult_mKK_d0bar", mKK, a1_mult_d0bar);  

  // RooProdPdf mult_peak_d0("mult_peak_d0", "mult_peak_d0", mult_mKK_d0, mult_mD0pi_d0);
  // RooProdPdf mult_peak_d0bar("mult_peak_d0bar", "mult_peak_d0bar", mult_mKK_d0bar, mult_mD0pi_d0bar);
  
  // //multibody & random pion
  // RooProdPdf mult_rnd_d0("mult_rnd_d0","mult_rnd_d0", mult_mKK_d0, bkg_mD0pi);
  // RooProdPdf mult_rnd_d0bar("mult_rnd_d0bar","mult_rnd_d0bar", mult_mKK_d0bar, bkg_mD0pi);

  RooProdPdf mult_peak_d0("mult_peak_d0", "mult_peak_d0", mult_mKK, mult_mD0pi_d0);
  RooProdPdf mult_peak_d0bar("mult_peak_d0bar", "mult_peak_d0bar", mult_mKK, mult_mD0pi_d0bar);
  
  //multibody & random pion
  RooProdPdf mult_rnd_d0("mult_rnd_d0","mult_rnd_d0", mult_mKK, bkg_mD0pi);
  RooProdPdf mult_rnd_d0bar("mult_rnd_d0bar","mult_rnd_d0bar", mult_mKK, bkg_mD0pi);

  /*----------------------- Ds+ -> K+ K- pi+ -------------------------*/
  //d0pi
  RooRealVar slope_ds("slope_ds", "Slope1 of Polynomial",7.49684e-01);
  RooRealVar A_slope_ds("A_{slope_ds}","",0);
  RooFormulaVar slope_ds_d0("slope_ds_d0","","(@0*(1 + @1))",RooArgList(slope_ds, A_slope_ds));
  RooFormulaVar slope_ds_d0bar("slope_ds_d0bar","","(@0*(1- @1))",RooArgList(slope_ds, A_slope_ds));

  RooChebychev ds_mD0pi_d0("ds_mD0pi_d0","Chebshev Polynomial", mD0pi,RooArgList(slope_ds_d0));
  RooChebychev ds_mD0pi_d0bar("ds_mD0pi_d0bar","Chebshev Polynomial", mD0pi,RooArgList(slope_ds_d0bar));

  //kk
  RooRealVar mu_0("#mu_{0}", "",-1.22819e-04);
  RooRealVar A_mu_0("A_{mu_0}","",0);
  RooFormulaVar mu_0_d0("mu_0_d0","","(@0*(1 + @1))",RooArgList(mu_0, A_mu_0));
  RooFormulaVar mu_0_d0bar("mu_0_d0bar","","(@0*(1- @1))",RooArgList(mu_0, A_mu_0));

  RooRealVar constant("constant", "",3.83319);
  RooFormulaVar mu_ds_d0("#mu_{ds_d0}", "@0+@1-@2",RooArgList(mu_0_d0,constant, mD0pi));
  RooFormulaVar mu_ds_d0bar("#mu_{ds_d0bar}", "@0+@1-@2",RooArgList(mu_0_d0bar,constant, mD0pi));
  
  RooRealVar sigma_ds("#sigma_{ds}", "",2.91096e-03);
  RooRealVar A_sigma_ds("A_{sigma_ds}","",0);
  RooFormulaVar sigma_ds_d0("sigma_ds_d0","","(@0*(1 + @1))",RooArgList(sigma_ds, A_sigma_ds));
  RooFormulaVar sigma_ds_d0bar("sigma_ds_d0bar","","(@0*(1- @1))",RooArgList(sigma_ds, A_sigma_ds));

  RooRealVar gamma_ds("#gamma_{ds}", "",0);
  RooRealVar A_gamma_ds("A_{gamma_ds}","",0);
  RooFormulaVar gamma_ds_d0("gamma_ds_d0","","(@0*(1 + @1))",RooArgList(gamma_ds, A_gamma_ds));
  RooFormulaVar gamma_ds_d0bar("gamma_ds_d0bar","","(@0*(1- @1))",RooArgList(gamma_ds, A_gamma_ds));

  RooRealVar delta_ds("#delta_{ds}", "",6.50303e-01);
  RooRealVar A_delta_ds("A_{delta_ds}","",0);
  RooFormulaVar delta_ds_d0("delta_ds_d0","","(@0*(1 + @1))",RooArgList(delta_ds, A_delta_ds));
  RooFormulaVar delta_ds_d0bar("delta_ds_d0bar","","(@0*(1- @1))",RooArgList(delta_ds, A_delta_ds));

  myRooJohnsonSU ds_mKK_d0("ds_mKK_d0","johnson_d0",mKK, mu_ds_d0,sigma_ds_d0,delta_ds_d0,gamma_ds_d0);
  myRooJohnsonSU ds_mKK_d0bar("ds_mKK_d0bar","johnson_d0bar",mKK, mu_ds_d0bar,sigma_ds_d0bar,delta_ds_d0bar,gamma_ds_d0bar);
 
  RooProdPdf ds_peak_d0("ds_peak_d0", "ds_peak_d0", ds_mD0pi_d0, Conditional(ds_mKK_d0, mKK));  
  RooProdPdf ds_peak_d0bar("ds_peak_d0bar", "ds_peak_d0bar", ds_mD0pi_d0bar, Conditional(ds_mKK_d0bar, mKK));

  /*------------------------C O M B I N A T O R I A L----------------------------*/

  RooRealVar c("c_{comb}", "Slope1 of Polynomial",-2.82122e-01, -1,1);
  RooRealVar A_c("A_{c}","",0);
  RooFormulaVar c_d0("c_d0","","(@0*(1 + @1))",RooArgList(c, A_c));
  RooFormulaVar c_d0bar("c_d0bar","","(@0*(1- @1))",RooArgList(c, A_c));

  RooChebychev comb_mKK_d0("chebpol_d0","Chebshev Polynomial", mKK,RooArgList(c_d0));
  RooChebychev comb_mKK_d0bar("chebpol_d0bar","Chebshev Polynomial", mKK,RooArgList(c_d0bar));
  
  RooProdPdf comb_bkg_d0("comb_bkg_d0", "comb_bkg_d0", comb_mKK_d0, bkg_mD0pi);
  RooProdPdf comb_bkg_d0bar("comb_bkg_d0bar", "comb_bkg_d0bar", comb_mKK_d0bar, bkg_mD0pi);

  /*------------------------Y I E L D S  F O R  A L L  C O M P O N E N T S----------------------------*/
  //yields
  RooRealVar sig_peak_yield("N_{sig_peak}","signal yield",2.46982e+05,1e1,1e7);
  RooRealVar sig_randompi_yield("N_{sig_rnd}","signal + random-pi yield",1.90369e+04,1e1,1e7); 
  RooRealVar kpi_peak_yield("N_{kpi_peak}","kpi yield",1.20757e+05,1e1,1e7);
  RooRealVar kpi_randompi_yield("N_{kpi_rnd}","kpi + random-pi yield",7.62380e+03,1e1,1e7);
  RooRealVar mult_peak_yield("N_{mult_peak}","multibody yield",7.79607e+04,1e1,1e7);
  RooRealVar mult_randompi_yield("N_{mult_rnd}","multibody + random-pi yield",8.83398e+03,1e1,1e5); 
  RooRealVar ds_peak_yield("N_{ds_peak}","ds yield",5.74284e+03,1e1,1e7);  
  RooRealVar comb_yield("N_{comb}","combinatorial yield",1.97739e+05,0.,1e7);
  
  //Asymmetries
  RooRealVar Araw_sig("A_{sig}","Araw",0,-1,1);
  RooRealVar Araw_sig_randompi("A_{sig_randompi}","Araw",0,-1,1);
  RooRealVar Araw_kpi("A_{kpi}","Araw",0,-1,1);
  RooRealVar Araw_kpi_randompi("A_{kpi_randompi}","Araw",0,-1,1);
  RooRealVar Araw_mult("A_{mult}","Araw",0,-1,1);
  RooRealVar Araw_mult_randompi("A_{mult_randompi}","Araw",0,-1,1);
  RooRealVar Araw_ds("A_{Ds}","Araw",0,-1,1);
  RooRealVar Araw_comb("A_{comb}","Araw",0,-1,1);
 

  //Yields for D0 sample: ND0 = N(1+A)/2
  RooFormulaVar sig_peak_yield_d0("ND0_{sig_peak}","(@0*(1 + @1))/2",RooArgList(sig_peak_yield,Araw_sig));
  RooFormulaVar sig_randompi_yield_d0("ND0_{sig_rnd}","(@0*(1 + @1))/2",RooArgList(sig_randompi_yield,Araw_sig_randompi));
  RooFormulaVar kpi_peak_yield_d0("ND0_{kpi_peak}","(@0*(1 + @1))/2",RooArgList(kpi_peak_yield,Araw_kpi));
  RooFormulaVar kpi_randompi_yield_d0("ND0_{kpi_rnd}","(@0*(1 + @1))/2",RooArgList(kpi_randompi_yield,Araw_kpi_randompi));
  RooFormulaVar mult_peak_yield_d0("ND0_{mult_peak}","(@0*(1 + @1))/2",RooArgList(mult_peak_yield,Araw_mult));
  RooFormulaVar mult_randompi_yield_d0("ND0_{mult_rnd}","(@0*(1 + @1))/2",RooArgList(mult_randompi_yield,Araw_mult_randompi));
  RooFormulaVar ds_peak_yield_d0("ND0_{ds_peak}","(@0*(1 + @1))/2",RooArgList(ds_peak_yield,Araw_ds));
  RooFormulaVar comb_yield_d0("ND0_{comb}","(@0*(1 + @1))/2",RooArgList(comb_yield,Araw_comb));

  
  //Yields for D0bar sample: ND0bar = N(1-A)/2
  RooFormulaVar sig_peak_yield_d0bar("ND0bar_{sig_peak}","(@0*(1 - @1))/2",RooArgList(sig_peak_yield,Araw_sig));
  RooFormulaVar sig_randompi_yield_d0bar("ND0bar_{sig_rnd}","(@0*(1 - @1))/2",RooArgList(sig_randompi_yield,Araw_sig_randompi));
  RooFormulaVar kpi_peak_yield_d0bar("ND0bar_{kpi_peak}","(@0*(1 - @1))/2",RooArgList(kpi_peak_yield,Araw_kpi));
  RooFormulaVar kpi_randompi_yield_d0bar("ND0bar_{kpi_rnd}","(@0*(1 - @1))/2",RooArgList(kpi_randompi_yield,Araw_kpi_randompi));
  RooFormulaVar mult_peak_yield_d0bar("ND0bar_{mult_peak}","(@0*(1 - @1))/2",RooArgList(mult_peak_yield,Araw_mult));
  RooFormulaVar mult_randompi_yield_d0bar("ND0bar_{mult_rnd}","(@0*(1 - @1))/2",RooArgList(mult_randompi_yield,Araw_mult_randompi));
  RooFormulaVar ds_peak_yield_d0bar("ND0bar_{ds_peak}","(@0*(1 - @1))/2",RooArgList(ds_peak_yield,Araw_ds));
  RooFormulaVar comb_yield_d0bar("ND0bar_{comb}","(@0*(1 - @1))/2",RooArgList(comb_yield,Araw_comb));
  
  
  /*------------------------C O M P L E T E  M O D E L----------------------------*/

  RooAddPdf model_d0("model_d0","",RooArgList(signal_peak_d0,signal_rnd_d0,kpi_peak_d0,kpi_rnd_d0,mult_peak_d0,mult_rnd_d0,ds_peak_d0,comb_bkg_d0), RooArgList(sig_peak_yield_d0,sig_randompi_yield_d0,kpi_peak_yield_d0,kpi_randompi_yield_d0,mult_peak_yield_d0, mult_randompi_yield_d0,ds_peak_yield_d0,comb_yield_d0));
  RooAddPdf model_d0bar("model_d0bar","",RooArgList(signal_peak_d0bar,signal_rnd_d0bar,kpi_peak_d0bar,kpi_rnd_d0bar,mult_peak_d0bar,mult_rnd_d0bar,ds_peak_d0bar,comb_bkg_d0bar), RooArgList( sig_peak_yield_d0bar, sig_randompi_yield_d0bar, kpi_peak_yield_d0bar,kpi_randompi_yield_d0bar,mult_peak_yield_d0bar, mult_randompi_yield_d0bar,ds_peak_yield_d0bar,comb_yield_d0bar));

  RooAddPdf signal_peak("signal_peak", "signal_peak", RooArgList(signal_peak_d0, signal_peak_d0bar),RooArgList(sig_peak_yield_d0, sig_peak_yield_d0bar));
  RooAddPdf signal_rnd("signal_rnd", "signal_rnd", RooArgList(signal_rnd_d0, signal_rnd_d0bar),RooArgList(sig_randompi_yield_d0,  sig_randompi_yield_d0bar));
  
  RooAddPdf kpi_peak("kpi_peak", "kpi_peak", RooArgList(kpi_peak_d0, kpi_peak_d0bar),RooArgList(kpi_peak_yield_d0, kpi_peak_yield_d0bar));
  RooAddPdf kpi_rnd("kpi_rnd", "kpi_rnd", RooArgList(kpi_rnd_d0, kpi_rnd_d0bar),     RooArgList(kpi_randompi_yield_d0, kpi_randompi_yield_d0bar ));
  
  RooAddPdf mult_peak("mult_peak", "mult_peak", RooArgList(mult_peak_d0, mult_peak_d0bar), RooArgList(mult_peak_yield_d0, mult_peak_yield_d0bar));
  RooAddPdf mult_rnd("mult_rnd", "mult_rnd", RooArgList(mult_rnd_d0, mult_rnd_d0bar),      RooArgList(mult_randompi_yield_d0, mult_randompi_yield_d0bar));
  
  RooAddPdf ds_peak("ds_peak", "ds_peak", RooArgList(ds_peak_d0, ds_peak_d0bar),RooArgList(ds_peak_yield_d0, ds_peak_yield_d0bar));

  RooAddPdf comb_bkg("comb_bkg", "comb_bkg", RooArgList(comb_bkg_d0, comb_bkg_d0bar),RooArgList(comb_yield_d0,comb_yield_d0bar));

  // RooAddPdf model("model","",RooArgList(signal_peak,signal_rnd, mult_peak,mult_rnd,kpi_peak, kpi_rnd,ds_peak, comb_bkg), RooArgList(sig_peak_yield,sig_randompi_yield,mult_peak_yield, mult_randompi_yield, kpi_peak_yield,kpi_randompi_yield,ds_peak_yield,comb_yield));

  //Define category to distinguish d0 and d0bar samples events
  RooCategory sample("sample","sample") ;
  sample.defineType("D0");
  sample.defineType("D0bar");

  //Construct combined dataset
  RooDataSet combData("combData","combined data",RooArgSet(mKK, mD0pi,weights),Index(sample),Import("D0",*dataxy_d0),Import("D0bar",*dataxy_d0bar),WeightVar("weights"));
  //Construct a simultaneous pdf using category sample as index
  RooSimultaneous simPdf("simPdf","simultaneous pdf",sample) ;
  simPdf.addPdf(model_d0,"D0") ;
  simPdf.addPdf(model_d0bar,"D0bar");

  //Perform the simultaneous fit
  simPdf.fitTo(combData,Strategy(2), SumW2Error(true));
  // simPdf.fitTo(combData,Strategy(2),AsymptoticError(true));

   /////////////////////////////////////////////////////////////////////////////////////////////
  // Plot "D0" slice of simultaneous pdf for mKK

  RooPlot *kk_d0 = mKK.frame(Title("D0"));
  combData.plotOn(kk_d0,Cut("sample==sample::D0"),Name("mKK_d0"));
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),ProjWData(sample,combData));
  // simPdf.paramOn(kk_d0,dataxy_d0);
  // kk_d0->SetXTitle("#it{m}(#it{K}^{#minus}#it{K}^{+}) [GeV/#it{c}^{2}]");  
  kk_d0->SetYTitle("Candidates per 3 MeV/#it{c}^{2}");
  kk_d0->GetYaxis()->SetTitleOffset(1.5);
  kk_d0->SetLabelOffset(99);
  // kk_d0->SetTitleOffset(99);
  
  RooHist* hpullx_d0_mkk = kk_d0->pullHist();
  hpullx_d0_mkk->SetFillStyle(1001);
  hpullx_d0_mkk->SetFillColor(1);
  for(int i=0;i<hpullx_d0_mkk->GetN();++i) hpullx_d0_mkk->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullplotx_d0_mkk = mKK.frame(Title(" ")) ;
  pullplotx_d0_mkk->addPlotable(hpullx_d0_mkk,"B");
  pullplotx_d0_mkk->SetXTitle("#it{m}(#it{K}^{#minus}#it{K}^{+}) [GeV/#it{c}^{2}]");
  pullplotx_d0_mkk->SetMinimum(-4.);
  pullplotx_d0_mkk->SetMaximum(4.);
  pullplotx_d0_mkk->GetXaxis()->SetLabelSize(0.1);
  pullplotx_d0_mkk->GetXaxis()->SetTitleSize(0.1);
  pullplotx_d0_mkk->GetYaxis()->SetLabelSize(0.07);
    
  TCanvas* c1 = new TCanvas("c1", "c1",600, 600);
  Double_t xlow, ylow, xup, yup;
  c1->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c1->Divide(1,2);
  TVirtualPad *upPad1a = c1->GetPad(1);
  upPad1a->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad1a = c1->GetPad(2);
  dwPad1a->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c1->Update();
  c1->cd(1);

  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(RooArgSet(model_d0)),ProjWData(sample,combData),Name("model_d0"));
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(RooArgSet(signal_peak_d0,signal_rnd_d0)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kRed),Name("signal_component_mkk_d0")) ;
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(RooArgSet(kpi_peak_d0,kpi_rnd_d0)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kGreen),Name("kpi_component_mkk_d0"));
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(RooArgSet(mult_peak_d0, mult_rnd_d0)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kBlack),Name("mult_component_mkk_d0"));
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(RooArgSet(ds_peak_d0)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kCyan),Name("ds_component_mkk_d0"));
  simPdf.plotOn(kk_d0,Slice(sample,"D0"),Components(comb_bkg_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kMagenta),Name("combinatorial_component_mkk_d0"));
  
  kk_d0->Draw();
  c1->cd(2);
  pullplotx_d0_mkk->Draw();
  c1->Update();
  // upPad1a->SetLogy(1);
  c1->cd(2);
  pullplotx_d0_mkk->Draw();
  c1->Update();


  // Plot "D0bar" slice of simultaneous pdf for mKK 
  RooPlot *kk_d0bar = mKK.frame(Title("D0bar"));
  combData.plotOn(kk_d0bar,Cut("sample==sample::D0bar"),Name("mKK_d0bar"));
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),ProjWData(sample,combData));
  // kk_d0bar->SetXTitle("#it{m}(#it{K}^{#minus}#it{K}^{+}) [GeV/#it{c}^{2}]");
  // kk_d0bar->GetXAxis()->SetLabelOffset(99);
  kk_d0bar->SetLabelOffset(99);
  kk_d0bar->SetYTitle("Candidates per 3 MeV/#it{c}^{2}");
  kk_d0bar->GetYaxis()->SetTitleOffset(1.5);
  kk_d0bar->GetYaxis()->SetTitleOffset(1.5);

  RooHist* hpullx_d0bar_mkk = kk_d0bar->pullHist();
  hpullx_d0bar_mkk->SetFillStyle(1001);
  hpullx_d0bar_mkk->SetFillColor(1);
  for(int i=0;i<hpullx_d0bar_mkk->GetN();++i) hpullx_d0bar_mkk->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullplotx_d0bar_mkk = mKK.frame(Title(" ")) ;
  pullplotx_d0bar_mkk->addPlotable(hpullx_d0bar_mkk,"B");
  pullplotx_d0bar_mkk->SetXTitle("#it{m}(#it{K}^{#minus}#it{K}^{+}) [GeV/#it{c}^{2}]");
  pullplotx_d0bar_mkk->SetMinimum(-4.);
  pullplotx_d0bar_mkk->SetMaximum(4.);
  pullplotx_d0bar_mkk->GetXaxis()->SetLabelSize(0.1);
  pullplotx_d0bar_mkk->GetXaxis()->SetTitleSize(0.1);
  pullplotx_d0bar_mkk->GetYaxis()->SetLabelSize(0.07);
    
  TCanvas* c2 = new TCanvas("c2", "c2",600, 600);
  c2->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c2->Divide(1,2);
  TVirtualPad *upPad1b = c2->GetPad(1);
  upPad1b->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad1b = c2->GetPad(2);
  dwPad1b->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c2->Update();
  c2->cd(1);
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(model_d0bar)),ProjWData(sample,combData),Name("model_d0bar"));
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(signal_peak_d0bar,signal_rnd_d0bar)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kRed),Name("signal_component_mkk_d0bar")) ;
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(kpi_peak_d0bar,kpi_rnd_d0bar)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kGreen),Name("kpi_component_mkk_d0bar"));
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(mult_peak_d0bar, mult_rnd_d0bar)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kBlack),Name("mult_component_mkk_d0bar"));
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(ds_peak_d0bar)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kCyan),Name("ds_component_mkk_d0bar"));
  simPdf.plotOn(kk_d0bar,Slice(sample,"D0bar"),Components(comb_bkg_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kMagenta),Name("combinatorial_component_mkk_d0bar"));
  
  kk_d0bar->Draw();
  c2->cd(2);
  pullplotx_d0bar_mkk->Draw();
  c2->Update();
  
  // Plot "D0" slice of simultaneous pdf for m(D0pi).
  RooPlot *d0pi_d0 = mD0pi.frame(Title("D0"));
  combData.plotOn(d0pi_d0,Cut("sample==sample::D0"),Name("mD0pi_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),ProjWData(sample,combData));
  // d0pi_d0->SetXTitle("#it{m}(#it{D}^{0}#it{#pi}_{s}^{+}) [GeV/#it{c}^{2}]");
  // d0pi_d0->GetXAxis()->SetLabelOffset(99);
  d0pi_d0->SetLabelOffset(99);
  d0pi_d0->SetYTitle("Candidates per 0.2 MeV/#it{c}^{2}");
  d0pi_d0->GetYaxis()->SetTitleOffset(1.5);
  d0pi_d0->GetYaxis()->SetTitleOffset(1.5);

    
  RooHist* hpully_d0_md0pi = d0pi_d0->pullHist();
  hpully_d0_md0pi->SetFillStyle(1001);
  hpully_d0_md0pi->SetFillColor(1);
  for(int i=0;i<hpully_d0_md0pi->GetN();++i) hpully_d0_md0pi->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullploty_d0_md0pi = mD0pi.frame(Title(" ")) ;
  pullploty_d0_md0pi->addPlotable(hpully_d0_md0pi,"B");
  pullploty_d0_md0pi->SetXTitle("#it{m}(#it{D}^{0}#it{#pi}_{s}^{+}) [GeV/#it{c}^{2}]");
  pullploty_d0_md0pi->SetMinimum(-4.);
  pullploty_d0_md0pi->SetMaximum(4.);
  pullploty_d0_md0pi->GetXaxis()->SetLabelSize(0.1);
  pullploty_d0_md0pi->GetXaxis()->SetTitleSize(0.1);
  pullploty_d0_md0pi->GetYaxis()->SetLabelSize(0.07);
    
  TCanvas* c3 = new TCanvas("c3", "c3",600, 600);
  c3->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c3->Divide(1,2);
  TVirtualPad *upPad3a = c3->GetPad(1);
  upPad3a->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad3a = c3->GetPad(2);
  dwPad3a->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c3->Update();
  c3->cd(1);
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(RooArgSet(model_d0)),ProjWData(sample,combData),Name("model_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(signal_peak_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kRed),Name("signal_peak_d0")) ;
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(kpi_peak_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kGreen),Name("kpi_peak_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(mult_peak_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kBlack),Name("mult_peak_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(ds_peak_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kCyan),Name("ds_peak_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(comb_bkg_d0),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kMagenta),Name("comb_d0pi_d0"));
  simPdf.plotOn(d0pi_d0,Slice(sample,"D0"),Components(RooArgSet(signal_rnd_d0,kpi_rnd_d0,mult_rnd_d0)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kOrange+2),Name("randompi_d0pi_d0"));
  
  d0pi_d0->Draw();
 
  c3->cd(2);
  pullploty_d0_md0pi->Draw();
  c3->Update();


  // Plot "D0bar" slice of simultaneous pdf for m(D0pi).
  RooPlot *d0pi_d0bar = mD0pi.frame(Title("D0bar"));
  combData.plotOn(d0pi_d0bar,Cut("sample==sample::D0bar"),Name("mD0pi_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),ProjWData(sample,combData));
  simPdf.paramOn(d0pi_d0bar,dataxy_d0bar);
  //d0pi_d0bar->SetXTitle("#it{m}(#it{D}^{0}#it{#pi}_{s}^{+}) [GeV/#it{c}^{2}]");
  // d0pi_d0bar->GetXAxis()->SetLabelOffset(99);
  d0pi_d0bar->SetLabelOffset(99);
  d0pi_d0bar->SetYTitle("Candidates per 0.2 MeV/#it{c}^{2}");
  d0pi_d0bar->GetYaxis()->SetTitleOffset(1.5);
  d0pi_d0bar->GetYaxis()->SetTitleOffset(1.5);

    
  RooHist* hpully_d0bar_md0pi = d0pi_d0bar->pullHist();
  hpully_d0bar_md0pi->SetFillStyle(1001);
  hpully_d0bar_md0pi->SetFillColor(1);
  for(int i=0;i<hpully_d0bar_md0pi->GetN();++i) hpully_d0bar_md0pi->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullploty_d0bar_md0pi = mD0pi.frame(Title(" ")) ;
  pullploty_d0bar_md0pi->addPlotable(hpully_d0bar_md0pi,"B");
  pullploty_d0bar_md0pi->SetXTitle("#it{m}(#it{D}^{0}#it{#pi}_{s}^{+}) [GeV/#it{c}^{2}]");
  pullploty_d0bar_md0pi->SetMinimum(-4.);
  pullploty_d0bar_md0pi->SetMaximum(4.);
  pullploty_d0bar_md0pi->GetXaxis()->SetLabelSize(0.1);
  pullploty_d0bar_md0pi->GetXaxis()->SetTitleSize(0.1);
  pullploty_d0bar_md0pi->GetYaxis()->SetLabelSize(0.07);


  TCanvas* c4 = new TCanvas("c4", "c4",600, 600);
  c4->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c4->Divide(1,2);
  TVirtualPad *upPad3b = c4->GetPad(1);
  upPad3b->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad3b = c4->GetPad(2);
  dwPad3b->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c4->Update();
  c4->cd(1);
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(model_d0bar)),ProjWData(sample,combData),Name("model_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(signal_peak_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kRed),Name("signal_peak_d0bar")) ;
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(kpi_peak_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kGreen),Name("kpi_peak_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(mult_peak_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kBlack),Name("mult_peak_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(ds_peak_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kCyan),Name("ds_peak_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(comb_bkg_d0bar),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kMagenta),Name("comb_d0pi_d0bar"));
  simPdf.plotOn(d0pi_d0bar,Slice(sample,"D0bar"),Components(RooArgSet(signal_rnd_d0bar,kpi_rnd_d0bar,mult_rnd_d0bar)),ProjWData(sample,combData),LineStyle(kDashed),LineColor(kOrange+2),Name("randompi_d0pi_d0bar"));
  
  d0pi_d0bar->Draw();

  c4->cd(2);
  pullploty_d0bar_md0pi->Draw();
  c4->Update();
}
